/*
 * Copyright (c) 2006 Charles S. Wilson
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR 
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <stdlib.h>
# include <stdarg.h>
# include <string.h>
# include <float.h>
#endif

#include <stdio.h>

#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if HAVE_MALLOC_H
# include <malloc.h>
#endif

#if HAVE_LOCALE_H
# include <locale.h>
#endif

#if HAVE_WINDOWS_H && HAVE_OPENCLIPBOARD
# define WIN32_LEAD_AND_MEAN
# define NOMINMAX
# include <windows.h>
#endif


#include "checkX.h"
#include "libiberty_decls.h"
#include "getopt_decls.h"
#include "util.h"

int opt_nogui = 0;
int opt_notty = 0;
int opt_debug = 0;
int opt_location = 0;
double opt_timeout = 0.5;
#if defined(SILENT)
int opt_silent = 1;
#else
int opt_silent = 0;
#endif

typedef struct s_opts_ { 
  char* displayname;
  char* appendpath;
  char* prependpath;
  char* replacepath;
  char* xlibname;
} s_opts;

void cleanup(s_opts* s)
{
  if (!s) return;
  if (s->displayname) { free(s->displayname); s->displayname = NULL; }
  if (s->appendpath)  { free(s->appendpath);  s->appendpath  = NULL; }
  if (s->prependpath) { free(s->prependpath); s->prependpath = NULL; }
  if (s->replacepath) { free(s->replacepath); s->replacepath = NULL; }
  if (s->xlibname)    { free(s->xlibname);    s->xlibname    = NULL; }
}
void dumpOpts(s_opts* s)
{
  if (s) {
    if (s->displayname) { debugMsg("displayname : '%s'", s->displayname); } 
    if (s->appendpath)  { debugMsg("appendpath  : '%s'", s->appendpath);  }
    if (s->prependpath) { debugMsg("prependpath : '%s'", s->prependpath); }
    if (s->replacepath) { debugMsg("replacepath : '%s'", s->replacepath); }
    if (s->xlibname)    { debugMsg("xlibname    : '%s'", s->xlibname);    }
  }
  debugMsg("opt_debug   : %d", opt_debug);
  debugMsg("opt_location: %d", opt_location);
  debugMsg("opt_silent  : %d", opt_silent);
  debugMsg("opt_nogui   : %d", opt_nogui);
  debugMsg("opt_notty   : %d", opt_notty);
  debugMsg("opt_timeout : %5.2f", opt_timeout);
}

int realMain(int argc, char* argv[])
{
  int i, c, count = 0;
  char* pathToX11Lib = NULL;
  s_opts sopt;
  char* p;

  int rc = 0;
  sopt.displayname = NULL;
  sopt.appendpath = NULL;
  sopt.prependpath = NULL;
  sopt.replacepath = NULL;
  sopt.xlibname = NULL;

  /***********************
   * Option parsing
   ***********************/
  while (1) {
    static struct option long_options[] =
    {
      {"help", no_argument, NULL, 'h'},
      {"version", no_argument, NULL, 'v'},
      {"display", required_argument, NULL, 'd'},
      {"location", no_argument, NULL, 'l'},
      {"appendpath", required_argument, NULL, 'a'},
      {"prependpath", required_argument, NULL, 'p'},
      {"replacepath", required_argument, NULL, 'r'},
      {"xlibname", required_argument, NULL, 'x'},
      {"timeout", required_argument, NULL, 't'},
#if (ENABLE_GUI == 1)
      {"nogui", no_argument, &opt_nogui, 1},
#endif
#if (ENABLE_TTY == 1)
      {"notty", no_argument, &opt_notty, 1},
#endif
      {"debug", no_argument, &opt_debug, 1},
#if defined(SILENT)
      {"no-silent", no_argument, &opt_silent, 0},
#else
      {"silent", no_argument, &opt_silent, 1},
#endif
      {0, 0, 0, 0}
    };

    static char* helpString = 
    "Usage: checkX [OPTION]...\n"
    "Determines if X is installed, Xserver is running on specified DISPLAY\n"
    "and will accept clients. Returns 0 if yes, nonzero otherwise\n\n"
    "Options:\n"
    "  -h|--help                  print this help message and exit\n"
    "  -v|--version               print version information and exit\n"
    "  -d|--display STRING        use STRING instead of $DISPLAY\n"
    "  -l|--location              print location of Xlib DLL on stdout\n"
    "  -a|--appendpath STRING     append STRING to value of $PATH (cumulative)\n"
    "  -p|--prependpath STRING    prepend STRING to value of $PATH (cumulative)\n"
    "  -r|--replacepath STRING    use STRING instead of $PATH when searching\n"
    "  -x|--xlibname STRING       use exactly STRING instead of fuzzy cygX11-n.dll search\n"
    "  -t|--timeout FLOAT         allow FLOAT seconds to connect with Xserver\n" 
    "                             defaults to 0.5, use 0.0 for Xlib's (safe, 12s) timeout\n"
#if (ENABLE_GUI == 1)
    "     --nogui                 disable informational popups\n"
#endif
#if (ENABLE_TTY == 1)
    "     --notty                 disable stderr messages\n"
#endif
    "     --debug                 turn on debugging messages\n"
#if defined(SILENT)
    "     --no-silent             allow error, warning, and info messages\n"
#else
    "     --silent                suppress error, warning, and info messages\n"
#endif
    "Note that -a defaults to '/usr/bin:/usr/X11R6/bin'.  To eliminate the default,\n"
    "use '-a \"\"'\n";

    static char* versionString = PACKAGE_STRING;

    int option_index = 0;
    c = getopt_long(argc, argv, "hvd:la:p:r:x:t:", long_options, &option_index);

    if (c == -1)
      break;

    switch (c) {
      case 0:
        /* If this option set a flag, do nothing else now. */
        if (long_options[option_index].flag != 0)
        break;
      case 'h':
        realMsg(helpString);
        cleanup(&sopt);
        return 1;
      case 'v':
        realMsg(versionString);
        cleanup(&sopt);
        return 1;
      case 'l':
        opt_location = 1;
        break;
      case 'd':
        if (sopt.displayname) { free(sopt.displayname); sopt.displayname = NULL; }
        sopt.displayname = strdup(optarg);
        if (!sopt.displayname) {
          errorMsg("out of memory");
          cleanup(&sopt);
          return 1;
        }
        break;
      case 'a':
        if (!sopt.appendpath) {
          sopt.appendpath = strdup(optarg);
          if (!sopt.appendpath) {
            errorMsg("out of memory");
            cleanup(&sopt);
            return 1;
          }
        }
        else {
          p = sopt.appendpath;
          sopt.appendpath = (char*) malloc(strlen(sopt.appendpath) + strlen(optarg) + 2);
          if (!sopt.appendpath) {
            errorMsg("out of memory");
            free(p);
            cleanup(&sopt);
            return 1;
          }
          strcpy(sopt.appendpath, p);
          strcat(sopt.appendpath, SEP_CHAR);
          strcat(sopt.appendpath, optarg);
        }
        break;
      case 'p':
        if (!sopt.prependpath) {
          sopt.prependpath = strdup(optarg);
          if (!sopt.prependpath) {
            errorMsg("out of memory");
            cleanup(&sopt);
            return 1;
          }
        }
        else {
          p = sopt.prependpath;
          sopt.prependpath = (char*) malloc(strlen(sopt.prependpath) + strlen(optarg) + 2);
          if (!sopt.prependpath) {
            errorMsg("out of memory");
            free(p);
            cleanup(&sopt);
            return 1;
          }
          strcpy(sopt.prependpath, optarg);
          strcat(sopt.prependpath, SEP_CHAR);
          strcat(sopt.prependpath, p);
        }
        break;
      case 'r':
        if (sopt.replacepath) { free(sopt.replacepath); sopt.replacepath = NULL; }
        sopt.replacepath = strdup(optarg);
        if (!sopt.replacepath) {
          errorMsg("out of memory");
          cleanup(&sopt);
          return 1;
        }
        break;
      case 'x':
        if (sopt.xlibname) { free(sopt.xlibname); sopt.xlibname = NULL; }
        sopt.xlibname = strdup(optarg);
        if (!sopt.xlibname) {
          errorMsg("out of memory");
          cleanup(&sopt);
          return 1;
        }
        break;
      case 't':
        opt_timeout = strtod(optarg, &p);
        if (optarg == p) {
          errorMsg("bad value specified for timeout: %s\n", optarg);
          cleanup(&sopt);
          return 1;
        }
        break;
      case '?':
        /* getopt_long already printed an error message. */
        cleanup(&sopt);
        return 1;
      default:
        errorMsg("unrecognized option");
        cleanup(&sopt);
        return 1;
    }
  }
  if ((opt_nogui || (!ENABLE_GUI)) && (opt_notty || (!ENABLE_TTY)))
    if (opt_debug)
      opt_notty = 0;

  /* propagate options to utils */
  set_tty_mode(!opt_notty);
  set_gui_mode(!opt_nogui);
  set_silent_mode(opt_silent);
  set_debug_mode(opt_debug);
  
  dumpOpts(&sopt);

  rc = checkX(sopt.prependpath,
              sopt.appendpath,
              sopt.replacepath,
              sopt.xlibname,
              sopt.displayname,
              opt_timeout,
              &pathToX11Lib);              

  cleanup(&sopt);
  if (rc != 0)
  {
    rc = 1;
  }
  else
  {
    if (opt_location && (rc == 0) && pathToX11Lib)
    {
      /* deliberately do NOT use message_() macros */
      fprintf(stdout, "%s", pathToX11Lib);
    }
  }

  if (pathToX11Lib) free(pathToX11Lib);
  debugMsg("returning with status %d", rc);
  return rc; /* exits with status of 0 if X available */
}

int WINAPI
WinMain (HINSTANCE hSelf, HINSTANCE hPrev, LPSTR cmdline, int nShow)
{
  char **argv, *p, **q;
  int argc;
  char bufArg0[MAX_PATH];
  char bufTemp[MAX_PATH];
  char* fullCmdLine;
  int rc, i;

  opt_debug = 0;
  opt_nogui = 0;
  opt_notty = 0;
#if defined(SILENT)
  opt_silent = 1;
#else
  opt_silent = 0;
#endif

  setlocale(LC_ALL, "C");

  debugMsg("orig cmdline = %s", cmdline);
  GetModuleFileName(NULL, bufTemp, sizeof(bufTemp));
  cygwin_conv_to_full_posix_path(bufTemp, bufArg0);

  fullCmdLine = (char*)malloc(strlen(bufArg0) + strlen(cmdline) + 4);  
  if (!fullCmdLine) {
    errorMsg("out of memory");
    return 1;
  }

  /* protect embedded spaces in bufArg0 */
  strcpy(fullCmdLine, "\"");
  strcat(fullCmdLine, bufArg0);
  strcat(fullCmdLine, "\" ");
  strcat(fullCmdLine, cmdline);

  debugMsg("new cmdline = %s", fullCmdLine);

  argv = buildargv(fullCmdLine);
  argc = countargv(argv);
  debugMsg("argc = %d", argc);
  if(argv) {
    q = argv;
    for (q = argv; *q != NULL; q++)
      debugMsg(": '%s'", *q);
  }

  for (i = 0; i < argc; i++) {
    debugMsg("argv[%d]: '%s'", i, argv[i]);
  }

  rc = realMain(argc, argv);
  freeargv(argv);

  infoMsg("Exiting with status %d", rc);
  // note that this will kill the detached thread if it's still running
  return rc;
}

