/*
 * Copyright (c) 2006 Charles S. Wilson
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR 
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <stdlib.h>
# include <stdarg.h>
# include <string.h>
# include <float.h>
#endif

#include <stdio.h>

#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if HAVE_MALLOC_H
# include <malloc.h>
#endif
#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#if HAVE_SYS_CYGWIN_H
# include <sys/cygwin.h>
#endif
#if HAVE_WINDOWS_H && HAVE_OPENCLIPBOARD
# define WIN32_LEAD_AND_MEAN
# define NOMINMAX
# include <windows.h>
#endif

#include "util.h"

int gui_mode = 1; /* enabled */
int tty_mode = 1; /* enabled */
int silent_mode = 1; /* silent */
int debug_mode = 0;  /* no debug */

int gui_is_allowed(void)  { return gui_mode;    }
int tty_is_allowed(void)  { return tty_mode;    }
int mode_is_silent(void)  { return silent_mode; }
int mode_is_debug(void)   { return debug_mode;  }

void set_gui_mode(int mode)    { gui_mode = mode;    }
void set_tty_mode(int mode)    { tty_mode = mode;    }
void set_silent_mode(int mode) { silent_mode = mode; }
void set_debug_mode(int mode)  { debug_mode = mode;  }

/*
 * This function attempts to locate a file which may be in any of
 * several directories. Unlike the original pfopen, it does not
 * return a FILE pointer to the opened file, but rather returns
 * the fully-qualified filename of the first match found. Returns
 * empty string if not found.
 */
char *pfopen(char *retval, const char *name, const char *dirs)
{
    char *ptr;
    char *tdirs;
    char returnval[MAX_PATH + FILENAME_MAX + 100];
    char *recursive_name;
    int foundit = FALSE;
    
    returnval[0] = '\0';

    if (dirs == NULL || dirs[0] == '\0')
        return NULL;

    if ((tdirs = malloc(strlen(dirs)+1)) == NULL)
        return NULL;

    strcpy(tdirs, dirs);

    for (ptr = strtok(tdirs, SEP_CHARS); (foundit == FALSE) && ptr != NULL;
         ptr = strtok(NULL, SEP_CHARS))
    {
       foundit = fileExists(returnval,ptr,name);
    }

    free(tdirs);
    if (!foundit)
        retval[0] = '\0';
    else
        strcpy(retval,returnval);
    return retval;
}
int fileExists(char* fullname, const char* path, const char* name)
{
   int retval = FALSE;
   FILE* file;
   size_t len;
   char work[FILENAME_MAX];
   char work2[MAX_PATH + FILENAME_MAX + 100];
   size_t wlen = sizeof(work);
   if (path != NULL)
   {
      strncpy(work, path, wlen);
      work[sizeof(work)-1] = 0;
      len = strlen(work);
      wlen = sizeof(work) - len;
      if (len && work[len-1] != '/' && work[len-1] != '\\')
      {    
         strncat(work, PATH_SEP_CHAR_STR, wlen);
         work[sizeof(work)-1] = 0;
         wlen = sizeof(work) - strlen(work);
      }
   }
   else
      work[0]='\0';
   
   strncat(work, name, wlen);
   work[sizeof(work)-1] = 0;
   wlen = sizeof(work) - strlen(work);
   cygwin_conv_to_posix_path(work, work2); 

#ifdef GORY_DEBUG
   debugMsg("looking for...\t%s",work2);
#endif

   file = fopen(work2, "rb");
   if (file != NULL)
   {
      if (fullname != NULL)
         strcpy(fullname,work2);
      retval = TRUE;
      fclose(file);
   }
   return retval;
}
void message_(int level, char* fmt, ...)
{
   char buf[10000];
   int j;
   va_list args;
   va_start(args, fmt);
   j = vsprintf(buf,fmt,args);
   va_end(args);
   if (ENABLE_TTY && (tty_is_allowed())) {
     if (level == 2) 
       fprintf(stderr, "checkX Error: %s\n", buf);
     else if (level == 1)
       fprintf(stderr, "checkX Warning: %s\n", buf);
     else if (level == 0)
       fprintf(stderr, "checkX Info: %s\n", buf);
     else
       fprintf(stdout, buf);
     return;
   }
   if (ENABLE_GUI && (gui_is_allowed())) {
     if (level == 2) 
       MessageBox(NULL, buf, "checkX Error", MB_ICONERROR);
     else if (level == 1)
       MessageBox(NULL, buf, "checkX Warning", MB_ICONWARNING);
     else if (level == 0)
       MessageBox(NULL, buf, "checkX Info", MB_ICONINFORMATION);
     else
       MessageBox(NULL, buf, "checkX", MB_ICONINFORMATION);
     return;
   }
}
void debug_(char* fmt, ...)
{
   char buf[10000];
   int j;
   va_list args;
   va_start(args, fmt);
   j = vsprintf(buf,fmt,args);
   va_end(args);

   if (ENABLE_TTY && (tty_is_allowed())) {
     fprintf(stderr, "checkX DEBUG: %s\n", buf);
     return;
   }
   if (ENABLE_GUI && (gui_is_allowed())) {
     MessageBox(NULL, buf, "checkX DEBUG", MB_ICONINFORMATION);
     return;
   }
}
int countargv(char** vector)
{
  char **scan;
  int rc = 0;
  if (vector != NULL)
    for (scan = vector; *scan != NULL; scan++)
      rc++;
  return rc;
}

